import { Duplex, DuplexOptions, TransformCallback } from "stream";
/**
 * Options for the GroupByStream.
 */
export interface GroupByStreamOptions<T> extends DuplexOptions {
    objectMode?: true;
    groupBy: (chunk: T) => string;
}
/**
 * A class that allows you to transform and stream data in parallel.
 */
export declare class GroupByStream<T> extends Duplex {
    private buffer;
    private readonly groupBy;
    /**
     * Creates a new instance of GroupBy with the given options.
     *
     * @param {GroupByStreamOptions<T>} options - The options for the GroupBy.
     */
    constructor(options: GroupByStreamOptions<T>);
    /**
     * Writes a chunk of data to the stream, groups it according to a specified function,
     * and executes the callback.
     *
     * @param {T} chunk - The data chunk to write to the stream.
     * @param {BufferEncoding} encoding - The encoding of the data.
     * @param {TransformCallback} callback - The callback function to be executed after writing the data.
     * @return {void}
     */
    _write(chunk: T, encoding: BufferEncoding, callback: TransformCallback): void;
    /**
     * Finalize the stream by draining the buffer and pushing any remaining chunks to the stream.
     *
     * @param {TransformCallback} callback - The callback to be called when the stream is finalized.
     * @return {void}
     */
    _final(callback: TransformCallback): void;
    /**
     * Reading is not supported since writer finishes first.
     *
     * @return {void}
     */
    _read(): void;
    /**
     * Groups a chunk of data based on the provided groupBy function and stores it in the buffer.
     *
     * @param {T} chunk - The data chunk to be grouped.
     * @return {void} This function does not return anything.
     */
    private _groupBy;
}
