import { Duplex, DuplexOptions, TransformCallback } from "stream";
/**
 * Options for the ParallelStream.
 */
export interface ParallelStreamOptions<TInput, TOutput> extends DuplexOptions {
    maxConcurrent: number;
    objectMode?: true;
    transform: (chunk: TInput) => Promise<TOutput>;
}
/**
 * A class that allows you to transform and stream data in parallel.
 */
export declare class ParallelStream<TInput, TOutput> extends Duplex {
    private queue;
    private buffer;
    private pool;
    private readonly maxConcurrent;
    private readonly transform;
    /**
     * Creates a new instance of ParallelStream with the given options.
     *
     * @param {ParallelStreamOptions<TInput, TOutput>} options - The options for the ParallelStream.
     */
    constructor(options: ParallelStreamOptions<TInput, TOutput>);
    /**
     * A method to write data to the stream, push the chunk to the queue, transform it, and then execute the callback.
     *
     * @param {TInput} chunk - The data chunk to write to the stream.
     * @param {BufferEncoding} encoding - The encoding of the data.
     * @param {TransformCallback} callback - The callback function to be executed after writing the data.
     * @return {void} This function does not return anything.
     */
    _write(chunk: TInput, encoding: BufferEncoding, callback: TransformCallback): void;
    /**
     * Asynchronously finalizes the stream by draining the queue and buffer, pushing any remaining chunks to the stream,
     * and calling the provided callback when complete. If the stream is unable to push a chunk, the chunk is placed back
     * into the buffer and a PushError is passed to the callback.
     *
     * @param {TransformCallback} callback - The callback to be called when the stream is finalized.
     * @return {Promise<void>} A promise that resolves when the stream is finalized.
     */
    _final(callback: TransformCallback): Promise<void>;
    /**
     * Pushes the ready chunks to the consumer stream since the buffer is empty or the size limit is reached.
     *
     * @param {number} size - The size parameter for controlling the read operation.
     * @return {void} This function does not return anything.
     */
    _read(size: number): void;
    /**
     * Loop through the pool and queue to process chunks, adding promises to the pool.
     */
    private _transform;
}
