import { DiscardingStreamEventEmitters, DiscardingStreamEventHandlers } from "../events/_index";
import { ObjectDuplex, ObjectDuplexOptions } from "./ObjectDuplex";
/**
 * @abstract
 * @class
 * Abstract class that allows you to emit discarded data in a stream adding support to discard events.
 * @extends ObjectDuplex
 * @template T
 * @example
 * ```typescript
 * class DiscardingStreamImplementation<T> extends DiscardingStream<T> {
 *     constructor(){
 *         super({objectMode: true});
 *     }
 *
 *     _write(chunk: T, encoding: BufferEncoding, callback: TransformCallback): void {
 *         this.emit("discard", chunk);
 *         callback();
 *     }
 *
 *     _final(callback: TransformCallback): void {
 *         this.push(null);
 *         callback();
 *     }
 *
 *     _read(): void {}
 * }
 * const stream:DiscardingStreamImplementation<string> = new DiscardingStreamImplementation();
 *
 * stream.write("data1"); //Discarded
 * stream.write("data2"); //Discarded
 * stream.write("data3"); //Discarded
 * stream.end();
 *
 * stream.on("discard", (chunk: string) => {
 *     console.log(``Discarded chunk: ${chunk}```);
 * });
 * ```
 * ```shell
 * >> Discarded chunk: data1
 * >> Discarded chunk: data2
 * >> Discarded chunk: data3
 * ```
 */
export declare abstract class DiscardingStream<T> extends ObjectDuplex {
    /**
     * @constructor
     * @param options {ObjectDuplexOptions} - The options for the GroupBy.
     */
    constructor(options: ObjectDuplexOptions);
    addListener<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
    emit<U extends keyof DiscardingStreamEventEmitters<T>>(event: U, ...args: Array<DiscardingStreamEventEmitters<T>[U]>): boolean;
    on<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
    once<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
    prependListener<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
    prependOnceListener<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
    removeListener<U extends keyof DiscardingStreamEventHandlers<T>>(event: U, listener: DiscardingStreamEventHandlers<T>[U]): this;
}
