"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Job = void 0;
const events_1 = require("events");
/**
 * @abstract
 * @class
 * Abstract base class for all jobs.
 * @extends EventEmitter
 * @example
 * ```typescript
 * export class JobImplementation extends Job {
 *     protected _steps() {
 *         return [new PassingStepFirst(), new PassingStepSecond()];
 *     }
 * }
 *
 * const job = new JobImplementation("My job");
 * job.on("stepStart", (step:step) => {
 *     console.log(`Starting step ${step.name}`);
 * })
 * job.run()
 *     .then(() => {
 *         console.log("Job completed successfully");
 *     })
 *     .catch((error) => {
 *         console.log("Job completed with errors");
 *     });
 * ```
 * ```shell
 * >> Starting step PassingStepFirst
 * >> Starting step PassingStepSecond
 * >> Job completed successfully
 * ```
 */
class Job extends events_1.EventEmitter {
    /**
     * @constructor
     * @param {string} name - The name to assign to the Step.
     */
    constructor(name) {
        super();
        this.name = name;
    }
    /**
     * Asynchronously runs the job by executing each step in sequence.
     * @return {Promise<void>} A Promise that resolves when all steps are successfully executed or rejects if an error occurs.
     */
    async run() {
        this.emit("start");
        const steps = this._steps();
        try {
            for (const step of steps) {
                this.emit("stepStart", step);
                await step.run();
                this.emit("stepEnd", step);
            }
            this.emit("end");
            return Promise.resolve();
        }
        catch (e) {
            const error = e;
            return Promise.reject(error);
        }
    }
    /**
     * Adds an event listener to the specified event type.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    addListener(event, listener) {
        return super.addListener(event, listener);
    }
    /**
     * Emits an event of the specified type to the listeners.
     * @param event {start | end | stepStart | stepEnd}
     * @param args {Array<JobEventEmitters>} Data to sent to the listeners depending on the event type
     * @returns  {boolean}
     */
    emit(event, ...args) {
        return super.emit(event, ...args);
    }
    /**
     * Adds an event listener to the specified event type.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    on(event, listener) {
        return super.on(event, listener);
    }
    /**
     * Adds a one time event listener to the specified event type.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    once(event, listener) {
        return super.once(event, listener);
    }
    /**
     * Adds an event listener to the specified event type to the beginning of the listeners array.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    prependListener(event, listener) {
        return super.prependListener(event, listener);
    }
    /**
     * Adds a one time event listener to the specified event type to the beginning of the listeners array.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    prependOnceListener(event, listener) {
        return super.prependOnceListener(event, listener);
    }
    /**
     * Removes an event listener to the specified event type.
     * @param event {start | end | stepStart | stepEnd}
     * @param listener {Function}
     * @returns {this} allowing to chain
     */
    removeListener(event, listener) {
        return super.removeListener(event, listener);
    }
}
exports.Job = Job;
