"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GroupByStream = void 0;
const stream_1 = require("stream");
const PushError_1 = require("./errors/PushError");
const defaultOptions = {
    objectMode: true
};
/**
 * A class that allows you to transform and stream data in parallel.
 */
class GroupByStream extends stream_1.Duplex {
    /**
     * Creates a new instance of GroupBy with the given options.
     *
     * @param {GroupByStreamOptions<T>} options - The options for the GroupBy.
     */
    constructor(options) {
        const opts = { ...defaultOptions, ...options };
        super(opts);
        this.buffer = new Map();
        this.groupBy = opts.groupBy;
    }
    /**
     * Writes a chunk of data to the stream, groups it according to a specified function,
     * and executes the callback.
     *
     * @param {T} chunk - The data chunk to write to the stream.
     * @param {BufferEncoding} encoding - The encoding of the data.
     * @param {TransformCallback} callback - The callback function to be executed after writing the data.
     * @return {void}
     */
    _write(chunk, encoding, callback) {
        this._groupBy(chunk);
        callback();
    }
    /**
     * Finalize the stream by draining the buffer and pushing any remaining chunks to the stream.
     *
     * @param {TransformCallback} callback - The callback to be called when the stream is finalized.
     * @return {void}
     */
    _final(callback) {
        try {
            Array.from(this.buffer.values()).forEach((group) => {
                if (!this.push(group)) {
                    throw new PushError_1.PushError();
                }
            });
            this.push(null);
            callback();
        }
        catch (error) {
            const e = error;
            callback(e);
        }
    }
    /**
     * Reading is not supported since writer finishes first.
     *
     * @return {void}
     */
    _read() {
        return;
    }
    /**
     * Groups a chunk of data based on the provided groupBy function and stores it in the buffer.
     *
     * @param {T} chunk - The data chunk to be grouped.
     * @return {void} This function does not return anything.
     */
    _groupBy(chunk) {
        const groupKey = this.groupBy(chunk);
        const group = this.buffer.get(groupKey) ?? [];
        group.push(chunk);
        this.buffer.set(groupKey, group);
    }
}
exports.GroupByStream = GroupByStream;
