"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractBatchEntityReaderStream = void 0;
const _index_1 = require("../interfaces/_index");
/**
 * @class
 * Class that enable to implement classes to read data in batches of a specified size in different types of data storage.
 * @extends ObjectReadable
 * @template T
 */
class AbstractBatchEntityReaderStream extends _index_1.ObjectReadable {
    /**
     * @constructor
     * @param {AbstractBatchEntityReaderStreamOptions} options - The options for the AbstractBatchEntityReaderStream.
     * @param [options.batchSize] {number} - The maximum number of elements in a batch.
     */
    constructor(options) {
        super(options);
        this.reading = false;
        this.buffer = [];
        this.batchSize = options.batchSize;
    }
    /**
     * Reads a batch of data from the data storage and pushes it to the consumer stream.
     * If the size parameter is not specified, it reads the number of entities specified in the `batchSize` option.
     * If the size parameter is specified, it reads the minimum of the size and the `batchSize` option.
     * If no data is available, it pushes null to the consumer stream to signal that the end of the stream has been reached.
     * If an error occurs while reading data, it emits an error event to the stream.
     *
     * @param {number} [size] - The size parameter for controlling the read operation.
     * @returns {Promise<void>} A promise that resolves when the data has been read and pushed to the consumer stream.
     */
    async _read(size) {
        if (this.reading)
            return;
        this.reading = true;
        try {
            const entities = await this.fetch(Math.min(size, this.batchSize));
            if (entities.length === 0) {
                this.push(null);
            }
            else {
                this.buffer.push(...entities);
                await this._flush();
            }
        }
        catch (error) {
            this.emit("error", error);
        }
        finally {
            this.reading = false;
        }
    }
    /**
     * Flushes the buffer by pushing its content to the consumer stream. If the consumer stream is not ready to receive data, it waits for the drain event and flushes the buffer again when it is emitted.
     * This function is recursive and will keep flushing the buffer until it is empty.
     *
     * @private
     * @returns {Promise<void>} A promise that resolves when the buffer is flushed.
     */
    async _flush() {
        while (this.buffer.length > 0) {
            const chunk = this.buffer.shift();
            if (!this.push(chunk)) {
                this.buffer.unshift(chunk);
                await new Promise((resolve) => this.once("drain", resolve));
            }
        }
        return Promise.resolve();
    }
    ;
}
exports.AbstractBatchEntityReaderStream = AbstractBatchEntityReaderStream;
