"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractBatchEntityWriterStream = void 0;
const _index_1 = require("../interfaces/_index");
/**
 * @class
 * Class that enable to implement classes to write data in batches of a specified size in different types of data storage.
 * @extends ObjectWritable
 * @template T
 */
class AbstractBatchEntityWriterStream extends _index_1.ObjectWritable {
    /**
     * @constructor
     * @param {AbstractBatchEntityWriterStreamOptions} options - The options for the AbstractBatchEntityWriterStream.
     * @param [options.batchSize] {number} - The maximum number of elements in a batch.
     */
    constructor(options) {
        super(options);
        this.buffer = [];
        this.batchSize = options.batchSize;
    }
    /**
     * A method to write data to the stream, push the chunk to the buffer, and execute the callback.
     *
     * @param {T} chunk - The data chunk to write to the stream.
     * @param {BufferEncoding} encoding - The encoding of the data.
     * @param {WriteCallback} callback - The callback function to be executed after writing the data.
     * @return {Promise<void>} This function does not return anything.
     */
    async _write(chunk, encoding, callback) {
        try {
            this.buffer.push(chunk);
            if (this.buffer.length >= this.batchSize) {
                await this._flush();
            }
            callback();
        }
        catch (error) {
            callback(error);
        }
    }
    /**
     * Finalizes the stream by pushing remaining data batches, handling errors,
     * and executing the final callback.
     *
     * @param {WriteCallback} callback - The callback function to be executed after finalizing the stream.
     * @return {Promise<void>} This function does not return anything.
     */
    async _final(callback) {
        try {
            await this._flush();
            callback();
        }
        catch (error) {
            callback(error);
        }
    }
    /**
     * Creates a batch of data from the buffer and flushes it to the storage.
     *
     * @private
     * @returns {Promise<void>}
     */
    async _flush() {
        if (this.buffer.length === 0)
            return Promise.resolve();
        const batch = [...this.buffer];
        this.buffer = [];
        try {
            await this.batchWrite(batch);
        }
        catch (error) {
            this.buffer.unshift(...batch);
            return Promise.reject(error);
        }
    }
}
exports.AbstractBatchEntityWriterStream = AbstractBatchEntityWriterStream;
