/**
 * @abstract
 * @class
 * Abstract base class for all steps.
 * @example
 * ```typescript
 * class StepImplementation extends Step {
    constructor(name: string = "MockPassingStep") {
        super(name);
    }

    protected _reader() {
        return new Readable({
            objectMode: true,
            read() {
                this.push("data");
                this.push(null);
            }
        });
    }

    protected _processors() {
        const opts: TransformOptions = {
            objectMode: true,
            transform(chunk: unknown, encoding: BufferEncoding, callback: TransformCallback) {
                this.push(chunk);
                callback();
            }
        };
        return [new Transform(opts), new Transform(opts)];
    }

    protected _writer() {
        return new Writable({
            objectMode: true,
            write(chunk: unknown, encoding: BufferEncoding, callback: TransformCallback) {
                callback();
            }
        });
    }
}
 * const step = new StepImplementation("StepImplementation");
 * step.run()
 *     .then(() => {
 *         console.log("Step completed successfully");
 *     })
 *     .catch((error) => {
 *         console.log("Step completed with errors");
 *     });
 * ```
 * ```shell
 * >> Step completed successfully
 * ```
 */
export class Step {
    /**
     * @constructor
     * @param {string} name - The name to assign to the Step.
     */
    constructor(name) {
        this.name = name;
    }
    /**
     * Executes the step by connecting streams, processing data, and listening for events.
     *
     * @return {Promise<void>} A Promise that resolves when the step execution is completed, and rejects if an error occurs.
     */
    run() {
        return new Promise((resolve, reject) => {
            // Reader
            const reader = this._reader();
            reader.once("error", (error) => {
                reject(error);
            });
            // Writer
            const writer = this._writer();
            writer.once("error", (error) => {
                reject(error);
            });
            // Assembly processors           
            let assembly = reader;
            for (const processor of this._processors()) {
                processor.once("error", (error) => {
                    reject(error);
                });
                assembly = assembly.pipe(processor);
            }
            // Assembly writer
            assembly.pipe(writer)
                .on("finish", () => {
                resolve();
            });
        });
    }
}
