import { Duplex } from "stream";
import { PushError } from "./errors/PushError";
const defaultOptions = {
    objectMode: true
};
/**
 * A class that allows you  stream data in batches of a specified size.
 */
export class FilterStream extends Duplex {
    /**
     * Initializes a new instance of the FilterStream class with the specified options.
     *
     * @param {FilterStreamOptions} options - The options for the FilterStream.
     */
    constructor(options) {
        const opts = { ...defaultOptions, ...options };
        super(opts);
        this.buffer = [];
        this._filter = opts.filter;
    }
    /**
     * A method to write data to the stream, filter the chunk and push it to the buffer or discard it, and execute the callback.
     *
     * @param {T} chunk - The data chunk to write to the stream.
     * @param {BufferEncoding} encoding - The encoding of the data.
     * @param {TransformCallback} callback - The callback function to be executed after writing the data.
     * @return {void} This function does not return anything.
     */
    _write(chunk, encoding, callback) {
        if (this._filter(chunk)) {
            this.buffer.push(chunk);
        }
        else {
            this.emit("discard", chunk);
        }
        callback();
    }
    /**
     * Finalizes the stream by pushing remaining data, handling errors,
     * and executing the final callback.
     *
     * @param {TransformCallback} callback - The callback function to be executed after finalizing the stream.
     * @return {void} This function does not return anything.
     */
    _final(callback) {
        while (this.buffer.length > 0) {
            const chunk = this.buffer.shift();
            if (!this.push(chunk)) {
                this.buffer.unshift(chunk);
                callback(new PushError());
                return;
            }
        }
        this.push(null);
        callback();
    }
    /**
     * Pushes the ready chunks to the consumer stream since the buffer is empty or the size limit is reached.
     *
     * @param {number} size - The size parameter for controlling the read operation.
     * @return {void} This function does not return anything.
     */
    _read(size) {
        while (this.buffer.length > 0 && size > 0) {
            const chunk = this.buffer.shift();
            if (!this.push(chunk)) {
                this.buffer.unshift(chunk);
            }
            size--;
        }
    }
    addListener(event, listener) {
        return super.addListener(event, listener);
    }
    emit(event, ...args) {
        return super.emit(event, ...args);
    }
    on(event, listener) {
        return super.on(event, listener);
    }
    once(event, listener) {
        return super.once(event, listener);
    }
    prependListener(event, listener) {
        return super.prependListener(event, listener);
    }
    prependOnceListener(event, listener) {
        return super.prependOnceListener(event, listener);
    }
    removeListener(event, listener) {
        return super.removeListener(event, listener);
    }
}
